#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <glib.h>

#include "callbacks.h"
#include "interface.h"
#include "valgrindifc.h"
#include "parser.h"

static gboolean gg_filter[GG_ENTRY_SIZE];
/*bzero(gg_filter,sizeof(gboolean)*GG_ENTRY_SIZE);*/
static void supp_set_list(GtkWidget* widget,GSList* data);

void
gnogrind_gui_init()
{
  gboolean def;
  /* initialize the dialog with default values */
  bzero(gg_filter,sizeof(gboolean)*GG_ENTRY_SIZE);
  gg_filter[GG_ENTRY_MISSMATCH]=
    gnome_config_get_bool_with_default("/gnogrind/gui/filtMISSMATCH=FALSE",&def);
  gg_filter[GG_ENTRY_WRITE]=
    gnome_config_get_bool_with_default("/gnogrind/gui/filtWRITE=FALSE",&def);
  gg_filter[GG_ENTRY_READ]=
    gnome_config_get_bool_with_default("/gnogrind/gui/filtREAD=FALSE",&def);
  gg_filter[GG_ENTRY_FREE]=
    gnome_config_get_bool_with_default("/gnogrind/gui/filtFREE=FALSE",&def);
  gg_filter[GG_ENTRY_CONDITION]=
    gnome_config_get_bool_with_default("/gnogrind/gui/filtCONDITION=FALSE",&def);
  gg_filter[GG_ENTRY_PARAM]=
    gnome_config_get_bool_with_default("/gnogrind/gui/filtPARAM=FALSE",&def);
  gg_filter[GG_ENTRY_UNINIT]=
    gnome_config_get_bool_with_default("/gnogrind/gui/filtUNINIT=FALSE",&def);
}

void
on_open (GtkWidget  *widget, gpointer user_data)
{
  appdlg dlg = create_appdlg();
  gboolean done = FALSE;
  gchar* appname=NULL;
  gchar* appargs=NULL;
  
  /* set current values */
  {
    char* txt = (char*)gnogrind_status(GG_GET_APPNAME);
    if(txt){
      gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(
		       GNOME_FILE_ENTRY(dlg.appname))),txt);
    }
  }

  {
    GPtrArray* arglist = (GPtrArray*)gnogrind_status(GG_GET_APPARGS);
    int i;
    for(i = 0; i < arglist->len; i++){
      char* thist = g_ptr_array_index(arglist,i);
      if(i){
	gtk_entry_append_text(GTK_ENTRY(dlg.appargs)," ");
      }
      /* if need braces ... */
      if(*thist == '\0' || strstr(thist," ")){
	gtk_entry_append_text(GTK_ENTRY(dlg.appargs),"\"");
	gtk_entry_append_text(GTK_ENTRY(dlg.appargs),thist);
	gtk_entry_append_text(GTK_ENTRY(dlg.appargs),"\"");
      } else {
	gtk_entry_append_text(GTK_ENTRY(dlg.appargs),thist);
      }
    }
  }

  /* while we don't get a correct input */
  while(done == FALSE){
    switch(gnome_dialog_run(GNOME_DIALOG(dlg.dlg))){
    case 0:
      /* OK */
      appname = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(dlg.appname),TRUE);
      if(appname == NULL){
	/* wrong file specified */
	break;
      }
      appargs = gtk_editable_get_chars(GTK_EDITABLE(dlg.appargs),0,-1);
      done = TRUE;
      break;
    default:
      done = TRUE;
      break;
    }
  }

  /* Now parse the args and appname */
  if(appname != NULL) {
    gnogrind_control(GG_SET_APPNAME,appname);
    gtk_widget_set_sensitive(gg_widgets[GG_START_BTN],TRUE);

    free(appname);
    if(appargs){
      /* split the appagrs */
      char* start = NULL;
      char* curr = appargs;
      GPtrArray* list = g_ptr_array_new();
      while(*curr){
	switch(*curr){
	case ' ':
	  *curr = '\0';
	  start = NULL;
	  break;
	case '"':
	  if(start == NULL){
	    start = ++curr;
	    g_ptr_array_add(list,start);
	    while(*curr) {
	      if (*curr == '"'){
		*curr = '\0';
		break;
	      }
	      curr++;
	    }
	  }
	  break;
	default:
	  if(start == NULL){
	    start = curr;
	    g_ptr_array_add(list,start);
	  }
	  break;
	}
	curr++;
      }
      gnogrind_control(GG_SET_APPARGS,list);
      g_ptr_array_free(list,TRUE);
      g_free(appargs);
    }
  }

  gtk_widget_destroy(dlg.dlg);
}

static void
on_openlog_activate(GtkWidget* widget,GtkWidget* dlg)
{
  char* filename = g_strdup( gtk_file_selection_get_filename (GTK_FILE_SELECTION(dlg)));
  gtk_widget_destroy(dlg);
  gnogrind_control(GG_SET_LOGFILE,filename);
  gnome_config_set_string("/gnogrind/gui/openlog",filename);
  g_free(filename);
}

void
on_openlog(GtkWidget *widget, gpointer user_data)
{
  gboolean def=TRUE;
  GtkWidget *dlg;
  char* initdir = gnome_config_get_string_with_default("/gnogrind/gui/openlog=NONE",&def);
  dlg = gtk_file_selection_new("Open log file...");
  if(def == FALSE){
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(dlg),initdir);
  }
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(dlg)->ok_button),
                      "clicked",
                      (GtkSignalFunc) on_openlog_activate,dlg);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(dlg)->cancel_button),
                             "clicked",
                             (GtkSignalFunc) gtk_widget_destroy,GTK_OBJECT(dlg));
  gtk_window_set_modal(GTK_WINDOW(dlg),TRUE);
  gtk_widget_show(dlg);
  g_free(initdir);
}

void
on_save_as (GtkWidget  *widget,  gpointer user_data)
{
}


void
on_exit_activate (GtkWidget  *widget, gpointer user_data)
{
  gnogrind_control(GG_STOP,NULL);
  gnogrind_finish();
  exit(0);
}


void
on_properties (GtkWidget  *widget, gpointer  user_data)
{
  propdlg dlg = create_propertybox();
  gboolean done = FALSE;
  char* valgrind = NULL;
  char* suppress = NULL;
  gboolean demangle = TRUE;
  guint depth = 4;
  gboolean limit = TRUE;

  /* set current values */
  gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(dlg.valgrind))),
		     (gchar*)gnogrind_status(GG_GET_VALGRIND));
  
  gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(dlg.suppfile))),
		     (gchar*)gnogrind_status(GG_GET_SUPPRESS));

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(dlg.depth),
			    (float)((guint)gnogrind_status(GG_GET_DEPTH)));

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg.demangle),
			       (gboolean)gnogrind_status(GG_GET_DEMANGLE));

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg.limit),
			       (gboolean)gnogrind_status(GG_GET_LIMIT));

  /* while we don't get a correct input */
  while(done == FALSE){
    switch(gnome_dialog_run(GNOME_DIALOG(dlg.dlg))){
    case 0:
      /* OK */
      valgrind = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(dlg.valgrind),TRUE);
      if(valgrind == NULL){
	/* wrong file specified */
	break;
      }
      suppress = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(dlg.suppfile),FALSE);
      depth = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dlg.depth));
      demangle = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dlg.demangle));
      limit = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dlg.limit));
      done = TRUE;
      break;
    default:
      done = TRUE;
      break;
    }
  }

  if(valgrind){
    /* set the settings */
    gnogrind_control(GG_SET_VALGRIND,valgrind);
    gnogrind_control(GG_SET_DEPTH,(gpointer)depth);
    gnogrind_control(GG_SET_DEMANGLE,(gpointer)demangle);
    gnogrind_control(GG_SET_SUPPRESS,suppress);
    gnogrind_control(GG_SET_LIMIT,(gpointer)limit);
    g_free(valgrind);
    g_free(suppress);
  }

  gtk_widget_destroy(dlg.dlg);
}


void
on_about (GtkWidget  *widget, gpointer  user_data)
{
}


void
on_start (GtkWidget *widget, gpointer user_data)
{
  /* clean the table */
  gtk_tree_remove_items(GTK_TREE(gg_widgets[GG_TREE_WDG]),
			gtk_container_children(GTK_CONTAINER(gg_widgets[GG_TREE_WDG])));
  gnogrind_control(GG_START,NULL);
}


void
on_stop (GtkWidget *widget, gpointer user_data)
{
  gnogrind_control(GG_STOP,NULL);
}

void
on_suppress (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg = create_suppdlg();
  char* suppfile = (char*)gnogrind_status(GG_GET_SUPPRESS);
  GSList* list = ggparser_readsupp(suppfile);
  supp_set_list(dlg,list);
  g_slist_free(list);
  gnome_dialog_run(GNOME_DIALOG(dlg));
}

void
on_filter (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg = create_filter();
  GtkWidget* cont = gtk_object_get_data(GTK_OBJECT(dlg),"vbox");


  {
    GList* children = gtk_container_children(GTK_CONTAINER(cont));
    int i;
    for(i=GG_ENTRY_NONE+1; i<GG_ENTRY_SUMMARY; i++){
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(children->data),gg_filter[i]);
      children = children->next;
    }
  }

  gtk_widget_show(dlg);
}

static void
gnogrindentry_on_destroy(GtkWidget* widget,gpointer dummy)
{
  gnogrind_entry_free((gnogrind_entry_t*)gtk_object_get_user_data(GTK_OBJECT(widget)));
}

void
gnogrind_handler(gnogrind_handler_t event,gpointer data)
{
  switch(event){
  case GG_HANDLE_STATUS:
    if(((int)data) == GG_RUNNING){
      gtk_widget_set_sensitive(gg_widgets[GG_START_BTN],FALSE);
      gtk_widget_set_sensitive(gg_widgets[GG_STOP_BTN],TRUE);
      gtk_widget_set_sensitive(gg_widgets[GG_OPEN_BTN],FALSE);
      gtk_widget_set_sensitive(gg_widgets[GG_OPENLOG_BTN],FALSE);
    } else {
      gtk_widget_set_sensitive(gg_widgets[GG_START_BTN],TRUE);
      gtk_widget_set_sensitive(gg_widgets[GG_STOP_BTN],FALSE);
      gtk_widget_set_sensitive(gg_widgets[GG_OPEN_BTN],TRUE);
      gtk_widget_set_sensitive(gg_widgets[GG_OPENLOG_BTN],TRUE);
    }
    break;
  case GG_HANDLE_ENTRY:
    {
      gnogrind_entry_t* entry = (gnogrind_entry_t*)data;
      GtkWidget *newitem = gtk_tree_item_new();
      GtkWidget *subtree = gtk_tree_new();
      int i = 0;
      char* label;
      GtkWidget* newitembox;
      GtkWidget* newitempxm;
      GtkWidget* newitemlbl;

      newitembox = gtk_hbox_new(FALSE,0);
      /*gtk_box_set_spacing(GTK_BOX(newitembox), 0);*/
      gtk_container_add(GTK_CONTAINER(newitem),newitembox);
      gtk_widget_show(newitembox);
      gtk_tree_append(GTK_TREE(gg_widgets[GG_TREE_WDG]),newitem);
      gtk_widget_show(newitem);
      
      /* choose an icon for this item */
      switch (entry->type){
      case GG_ENTRY_MISSMATCH:
      case GG_ENTRY_CONDITION:
	newitempxm = gtk_pixmap_new(GNOME_PIXMAP(gg_pixmaps[GG_PIXMAP_WARNING])->pixmap,
				    GNOME_PIXMAP(gg_pixmaps[GG_PIXMAP_WARNING])->mask);
	break;
      case GG_ENTRY_FREE:
      case GG_ENTRY_READ:
      case GG_ENTRY_WRITE:
      case GG_ENTRY_UNINIT:
	newitempxm = gtk_pixmap_new(GNOME_PIXMAP(gg_pixmaps[GG_PIXMAP_ERROR])->pixmap,
				    GNOME_PIXMAP(gg_pixmaps[GG_PIXMAP_ERROR])->mask);
	break;
      case GG_ENTRY_SUMMARY:
	newitempxm = gtk_pixmap_new(GNOME_PIXMAP(gg_pixmaps[GG_PIXMAP_INFO])->pixmap,
				    GNOME_PIXMAP(gg_pixmaps[GG_PIXMAP_INFO])->mask);
	break;
      default:
	newitempxm = gtk_pixmap_new(GNOME_PIXMAP(gg_pixmaps[GG_PIXMAP_UNKNOWN])->pixmap,
				    GNOME_PIXMAP(gg_pixmaps[GG_PIXMAP_UNKNOWN])->mask);
	break;
      }
      gtk_box_pack_start(GTK_BOX(newitembox),newitempxm,FALSE,FALSE,0);
      /* add label */
      gtk_box_pack_start(GTK_BOX(newitembox),newitemlbl=gtk_label_new(entry->header),
			 FALSE,FALSE,0);
      gtk_widget_show(newitempxm);
      gtk_widget_show(newitemlbl);

      /* add stack data under this entry */
      gtk_tree_item_set_subtree(GTK_TREE_ITEM(newitem),subtree);
      if(entry->data->len){
	while((label=g_ptr_array_index(entry->data,i)) != NULL){
	  GtkWidget* rep = gtk_tree_item_new_with_label(label); 
	  gtk_tree_append(GTK_TREE(subtree),rep);
	  gtk_widget_show(rep);
	  i++;
	}
      }
      gtk_object_set_user_data(GTK_OBJECT(newitem),entry);
      gtk_signal_connect(GTK_OBJECT(newitem),"destroy",gnogrindentry_on_destroy,NULL);
    }
    break;
  default:
    break;
  }
}

void
suppitem_on_destroy(GtkWidget* widget,gpointer dummy)
{
  gnogrind_supp_t* supp = gtk_object_get_user_data(GTK_OBJECT(widget));
  gnogrind_supp_free(supp);
}

void
supp_on_select(GtkWidget* widget,GtkListItem* item,gpointer user_data)
{
  GtkWidget* dlg = (GtkWidget*)user_data;
  gnogrind_supp_t* supp = (gnogrind_supp_t*)gtk_object_get_user_data(GTK_OBJECT(item));
  GtkEntry* name = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(dlg),"name"));
  GtkCombo* type = GTK_COMBO(gtk_object_get_data(GTK_OBJECT(dlg),"type"));
  GtkText*  stack = GTK_TEXT(gtk_object_get_data(GTK_OBJECT(dlg),"stack"));
  gtk_entry_set_text(name,supp->name);
  gtk_entry_set_text(GTK_ENTRY(type->entry),gg_supp_types[supp->type]);
  gtk_editable_delete_text(GTK_EDITABLE(stack),0,-1);
  {
    int i;
    char* item;
    for(i=0;i<supp->stack->len;i++){
      item = ((gnogrind_stack_t*)g_ptr_array_index(supp->stack,i))->value;
      if(i){
	gtk_text_insert(stack,NULL,NULL,NULL,"\n",-1);
      }
      gtk_text_insert(stack,NULL,NULL,NULL,item,-1);
    }
  }
  gtk_widget_set_sensitive(GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(dlg),"add")),FALSE);
}

static void
supp_set_list(GtkWidget* widget,GSList* data)
{
  GSList* iter = data;
  GtkList* list = GTK_LIST(gtk_object_get_data(GTK_OBJECT(widget),"list"));
  GtkListItem* item;
  gnogrind_supp_t* supp;

  gtk_list_clear_items(list,0,-1);
  while(iter){
    supp = (gnogrind_supp_t*)iter->data;
    item = GTK_LIST_ITEM(gtk_list_item_new_with_label(supp->name));
    gtk_object_set_user_data(GTK_OBJECT(item),supp);
    gtk_widget_show(GTK_WIDGET(item));
    gtk_container_add(GTK_CONTAINER(list),GTK_WIDGET(item));
    gtk_signal_connect (GTK_OBJECT (item), "destroy",
			GTK_SIGNAL_FUNC (suppitem_on_destroy),NULL);
    iter = iter->next;
  }

  if(data){
    gtk_list_select_item(list,0);
  }
}

void
supp_on_add(GtkWidget* widget,GtkWidget* dlg)
{
  GtkEntry* name = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(dlg),"name"));
  GtkCombo* type = GTK_COMBO(gtk_object_get_data(GTK_OBJECT(dlg),"type"));
  GtkText*  stack = GTK_TEXT(gtk_object_get_data(GTK_OBJECT(dlg),"stack"));
  GtkList*  list = GTK_LIST(gtk_object_get_data(GTK_OBJECT(dlg),"list"));
  char* nametext;
  char* stacktext;
  gnogrind_supptype_t typetext;
  char* delme[3];
  char** lines;
  gnogrind_supp_t* supp = NULL;

  /* find if this entry is ok */
  nametext = g_strstrip(delme[0]=g_strdup(gtk_entry_get_text(name)));
  stacktext = g_strstrip(delme[1]=gtk_editable_get_chars(GTK_EDITABLE(stack),0,-1));
  delme[2] = g_strdup(gtk_entry_get_text(GTK_ENTRY(type->entry)));
  for(typetext=GG_SUPP_NONE+1; typetext<GG_SUPP_SIZE; typetext++){
    if(strcmp(gg_supp_types[typetext],delme[2]) == 0){
      break;
    }
  }
  lines = g_strsplit(stacktext,"\n",-1);
  if(*nametext == '\0' || *stacktext == '\0'){
    goto errret;
  }
  {
    int i=0;
    while(lines[i]){
      char* thisline = g_strstrip(lines[i]);
      if(typetext != GG_SUPP_PARAM && strncmp(thisline,"obj:",4) &&
	 strncmp(thisline,"fun:",4)) {
	goto errret;
      }
      i++;
    }
    if(i == 0){
      goto errret;
    }
  }
  
  /* create the entry or clean old one...*/
  supp = gnogrind_supp_new();
  supp->name = g_strdup(nametext);
  supp->type = typetext;
  {
    int i = 0;
    while(lines[i]){
      gnogrind_stack_t* st = gnogrind_stack_new();
      if(strncmp(lines[i],"obj:",4) == 0){
	st->type = GG_STACK_OBJ;
      } else {
	st->type = GG_STACK_FUN;
      }
      st->value = g_strdup(lines[i]);
      g_ptr_array_add(supp->stack,st);
      i++;
    }
  }
  /* find if there is already and entry like that */
  {
    GList* supps = gtk_container_children(GTK_CONTAINER(list));
    while(supps){
      gnogrind_supp_t* s = (gnogrind_supp_t*)
	gtk_object_get_user_data(GTK_OBJECT(supps->data));
      if(strcmp(s->name,nametext) == 0){
	gnogrind_supp_free(s);
	gtk_object_set_user_data(GTK_OBJECT(supps->data),supp);
	break;
      }
      supps = supps->next;
    }
  }

  
  /* add/replace */
 errret:
  {
    int i = 0;
    for(i=0;i<3;i++){
      g_free(delme[i]);
    }
    g_strfreev(lines);
  }
}

void
supp_on_remove(GtkWidget* widget,GtkWidget* dlg)
{
  GtkList*  list = GTK_LIST(gtk_object_get_data(GTK_OBJECT(dlg),"list"));
  /* take selected row and remove it*/
  GList* sel = g_list_copy(list->selection);
  gtk_list_remove_items(list,sel);
  g_list_free(sel);
}

void
supp_on_save(GtkWidget* widget,GtkWidget* dlg)
{
  GtkList*  list = GTK_LIST(gtk_object_get_data(GTK_OBJECT(dlg),"list"));
  /* go over all entries and collect data */
  GList* entry = gtk_container_children(GTK_CONTAINER(list));
  /* now save it */
  GSList* data = NULL;
  while(entry){
    data = g_slist_append(data,gtk_object_get_user_data(GTK_OBJECT(entry->data)));
    entry = entry->next;
  }
  
  if (ggparser_writesupp((char*)gnogrind_status(GG_GET_SUPPRESS),data)) {
  }
  g_slist_free(data);
}

void
supp_on_changed(GtkWidget* widget,GtkWidget* dlg)
{
  gtk_widget_set_sensitive(GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(dlg),"add")),TRUE);
}

/* fileter callbacks */
void
filter_on_apply(GtkWidget* widget,GtkWidget* dlg)
{
  GtkWidget* cont = gtk_object_get_data(GTK_OBJECT(dlg),"vbox");

  /* initialize the dialog with default values */
  {
    GList* children = gtk_container_children(GTK_CONTAINER(cont));
    int i;
    for(i=GG_ENTRY_NONE+1; i<GG_ENTRY_SUMMARY; i++){
      gg_filter[i]=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(children->data));
      children = children->next;
    }
  }  
  gnome_config_set_bool("/gnogrind/gui/filtMISSMATCH",gg_filter[GG_ENTRY_MISSMATCH]);
  gnome_config_set_bool("/gnogrind/gui/filtWRITE",gg_filter[GG_ENTRY_WRITE]);
  gnome_config_set_bool("/gnogrind/gui/filtREAD",gg_filter[GG_ENTRY_READ]);
  gnome_config_set_bool("/gnogrind/gui/filtFREE",gg_filter[GG_ENTRY_FREE]);
  gnome_config_set_bool("/gnogrind/gui/filtCONDITION",gg_filter[GG_ENTRY_CONDITION]);
  gnome_config_set_bool("/gnogrind/gui/filtPARAM",gg_filter[GG_ENTRY_PARAM]);
  gnome_config_set_bool("/gnogrind/gui/filtUNINIT",gg_filter[GG_ENTRY_UNINIT]);

  /* now go over all entries in the list and show/hide respectively...*/
  {
    GList* list = gtk_container_children(GTK_CONTAINER(gg_widgets[GG_TREE_WDG]));
    gnogrind_entry_t* entry;
    while(list){
      entry = gtk_object_get_user_data(GTK_OBJECT(list->data));
      if(gg_filter[entry->type]){
	gtk_widget_hide(GTK_WIDGET(list->data));
      } else {
	gtk_widget_show(GTK_WIDGET(list->data));
      }
      list = list->next;
    }
  }
}

void
filter_on_ok(GtkWidget* widget,GtkWidget* dlg)
{
  filter_on_apply(widget,dlg);
  gtk_widget_destroy(dlg);
}
