#ifndef _VALGRINDIFC_H
#define _VAGRINDIFC_H

typedef enum {
  GG_START,
  GG_STOP,
  GG_SET_VALGRIND,
  GG_SET_APPNAME,
  GG_SET_APPARGS,
  GG_SET_DEPTH,
  GG_SET_DEMANGLE,
  GG_SET_SUPPRESS,
  GG_SET_LIMIT,
  GG_SET_LOGFILE
} gnogrind_control_t; 

typedef enum {
  GG_RUNNING,
  GG_STOPPED,
  GG_GET_APPNAME,
  GG_GET_APPARGS,
  GG_GET_VALGRIND,
  GG_GET_DEPTH,
  GG_GET_DEMANGLE,
  GG_GET_SUPPRESS,
  GG_GET_LIMIT
} gnogrind_status_t;

typedef enum {
  GG_HANDLE_STATUS,
  GG_HANDLE_ENTRY
} gnogrind_handler_t;

typedef enum {
  GG_ENTRY_NONE,
  GG_ENTRY_MISSMATCH,
  GG_ENTRY_WRITE,
  GG_ENTRY_READ,
  GG_ENTRY_FREE,
  GG_ENTRY_CONDITION,
  GG_ENTRY_PARAM,
  GG_ENTRY_UNINIT,
  GG_ENTRY_SUMMARY,
  GG_ENTRY_SIZE
} gnogrind_entrytype_t;

extern const char* gg_entry_types[GG_ENTRY_SIZE];

/* entry read from valgrind */
typedef struct {
  gnogrind_entrytype_t type;
  char* header;
  GPtrArray* data;
} gnogrind_entry_t;

/* entry read from suppressions file */
typedef enum {
  GG_SUPP_NONE,
  GG_SUPP_VALUE1,
  GG_SUPP_VALUE2,
  GG_SUPP_VALUE4,
  GG_SUPP_VALUE8,
  GG_SUPP_ADDR1,
  GG_SUPP_ADDR2,
  GG_SUPP_ADDR4,
  GG_SUPP_ADDR8,
  GG_SUPP_COND,
  GG_SUPP_PARAM,
  GG_SUPP_PTHREAD,
  GG_SUPP_FREE,
  GG_SUPP_SIZE
} gnogrind_supptype_t;

extern const char* gg_supp_types[GG_SUPP_SIZE];

typedef enum {
  GG_STACK_NONE,
  GG_STACK_OBJ,
  GG_STACK_FUN,
  GG_STACK_SIZE
}  gnogrind_stacktype_t;

extern const char* gg_stack_types[GG_STACK_SIZE];

typedef struct {
  gnogrind_stacktype_t type;
  gchar* value;
} gnogrind_stack_t;

typedef struct {
  gnogrind_supptype_t type;
  gchar* name;
  GPtrArray* stack;
} gnogrind_supp_t;

extern int  gnogrind_init(int argc,char* argv[]);
extern void gnogrind_finish();

extern gpointer gnogrind_status(gnogrind_status_t status);
extern int  gnogrind_control(gnogrind_control_t control , gpointer value);
extern void gnogrind_handler(gnogrind_handler_t event, gpointer data);

extern gnogrind_entry_t* gnogrind_entry_new();
extern void gnogrind_entry_free(gnogrind_entry_t* entry);

extern gnogrind_stack_t* gnogrind_stack_new();
extern void gnogrind_stack_free(gnogrind_stack_t* stack);

extern gnogrind_supp_t* gnogrind_supp_new();
extern void gnogrind_supp_free(gnogrind_supp_t* supp);
#endif
